==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/MixerSettingsDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.model.AudioMixer.*;
import static org.audiochain.ui.gui.CommonComponentFactory.*;
import static org.audiochain.ui.gui.Dialog.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.audiochain.io.AudioIOException;
import org.audiochain.model.AudioMixer;
import org.audiochain.model.HardwareMixer;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.GlobalGuiContext;
import org.audiochain.ui.gui.LightEmittingDiodeCheckBox;
import org.audiochain.ui.gui.NotificationDialog;

public class MixerSettingsDialog extends NotificationDialog {
	private JPanel mainPanel;
	private JLabel systemLatencyLabel;
	private JTextField systemLatencyField;
	private JLabel linePreinitializationLabel;
	private JCheckBox linePreinitializationCheckBox;
	private JLabel automaticMixerDetectionLabel;
	private JCheckBox automaticMixerDetectionCheckBox;
	private JLabel selectedPlaybackMixerLabel;
	private JComboBox selectedPlaybackMixerComboBox;
	private JLabel selectedCaptureMixerLabel;
	private JComboBox selectedCaptureMixerComboBox;
	private JLabel playbackBufferSizeLabel;
	private JTextField playbackBufferSizeField;
	private JLabel captureBufferSizeLabel;
	private JTextField captureBufferSizeField;

	public MixerSettingsDialog(Object caller) {
		super(caller);
		init();
	}

	protected void init() {
		if (!Dialog.okCancel(caller, "Mixer Settings Notification", "Please assert that there is no mixer in use by any playback or capture process!")) {
			return;
		}
		
		initComponents();
		initValues();
		registerListeners();
		createLayout();

		if (isButtonPressed(caller, "Audio Mixer Settings", mainPanel, applyButton)) {
			AudioMixer mixer = AudioMixer.getInstance();
			boolean autoDetection = automaticMixerDetectionCheckBox.isSelected();
			mixer.setAutomaticMixerDetection(autoDetection);
			mixer.setLinePreinitialization(linePreinitializationCheckBox.isSelected());
			mixer.setSystemLatencyInFrames(Integer.parseInt(systemLatencyField.getText().trim()));
			mixer.setPlaybackBufferSizeInSamples(Integer.parseInt(playbackBufferSizeField.getText().trim()));
			mixer.setCaptureBufferSizeInSamples(Integer.parseInt(captureBufferSizeField.getText().trim()));
			if (!autoDetection) {
				mixer.setSelectedPlaybackMixer((HardwareMixer) selectedPlaybackMixerComboBox.getSelectedItem());
				mixer.setSelectedCaptureMixer((HardwareMixer) selectedCaptureMixerComboBox.getSelectedItem());
			}
			mixer.store();
		}
	}

	private void initComponents() {
		mainPanel = new JPanel();
		mainPanel.setOpaque(false);
		applyButton = new JButton("Apply Changes");
		systemLatencyLabel = new JLabel("General System Latency (Frames):");
		systemLatencyField = new JTextField(4);
		linePreinitializationLabel = new JLabel("Initialize Audio Lines On Startup:");
		linePreinitializationCheckBox = new LightEmittingDiodeCheckBox();
		automaticMixerDetectionLabel = new JLabel("Automatic Mixer Detection:");
		automaticMixerDetectionCheckBox = new LightEmittingDiodeCheckBox();
		selectedPlaybackMixerLabel = new JLabel("Playback Mixer:");
		selectedPlaybackMixerComboBox = createComboBox(getAvailablePlaybackMixers());
		selectedCaptureMixerLabel = new JLabel("Recording Mixer:");
		selectedCaptureMixerComboBox = createComboBox(getAvailableCaptureMixers());
		playbackBufferSizeLabel = new JLabel("Playback Buffer Size (Samples):");
		playbackBufferSizeField = new JTextField(5);
		playbackBufferSizeField.setName("Playback Buffer Size");
		captureBufferSizeLabel = new JLabel("Recording Buffer Size (Samples):");
		captureBufferSizeField = new JTextField(5);
		captureBufferSizeField.setName("Capture Buffer Size");
	}

	private void initValues() {
		AudioMixer mixer = AudioMixer.getInstance();
		systemLatencyField.setText(String.valueOf(mixer.getSystemLatencyInFrames()));
		linePreinitializationCheckBox.setSelected(mixer.isLinePreinitialization());
		boolean automaticMixerDetection = mixer.isAutomaticMixerDetection();
		automaticMixerDetectionCheckBox.setSelected(automaticMixerDetection);

		if (automaticMixerDetection) {
			selectedPlaybackMixerComboBox.setSelectedItem(mixer.getDetectedPlaybackMixer());
			selectedCaptureMixerComboBox.setSelectedItem(mixer.getDetectedCaptureMixer());
		} else {
			try {
				HardwareMixer selectedPlaybackMixer = mixer.getSelectedPlaybackMixer();
				if (selectedPlaybackMixer != null) {
					selectedPlaybackMixerComboBox.setSelectedItem(selectedPlaybackMixer);
				}
				HardwareMixer selectedCaptureMixer = mixer.getSelectedCaptureMixer();
				if (selectedCaptureMixer != null) {
					selectedCaptureMixerComboBox.setSelectedItem(selectedCaptureMixer);
				}
			} catch (AudioIOException ex) {
				Dialog.message(MixerSettingsDialog.this, "Hardware Mixer Selection Error", ex);
			}
		}

		selectedPlaybackMixerComboBox.setEnabled(!automaticMixerDetectionCheckBox.isSelected());
		selectedCaptureMixerComboBox.setEnabled(!automaticMixerDetectionCheckBox.isSelected());

		playbackBufferSizeField.setText(String.valueOf(mixer.getPlaybackBufferSizeInSamples()));
		captureBufferSizeField.setText(String.valueOf(mixer.getCaptureBufferSizeInSamples()));
	}

	private void registerListeners() {
		systemLatencyField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			private void checkSystemLatencyValue() {
				putNotification(systemLatencyField, null);
				try {
					String text = systemLatencyField.getText();
					Integer.parseInt(text.trim());
				} catch (NumberFormatException e) {
					putNotification(systemLatencyField, systemLatencyLabel.getText() + " must have a numeric (integral) value: " + e.getMessage());
				}
			}
		});

		ActionListener mixerTestAction = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (automaticMixerDetectionCheckBox.isSelected()) {
					putNotification(selectedPlaybackMixerComboBox, null);
					return;
				}
				HardwareMixer selectedPlaybackMixer = (HardwareMixer) selectedPlaybackMixerComboBox.getSelectedItem();
				HardwareMixer selectedCaptureMixer = (HardwareMixer) selectedCaptureMixerComboBox.getSelectedItem();
				if (selectedPlaybackMixer == null || selectedCaptureMixer == null) {
					putNotification(selectedPlaybackMixerComboBox, "The Mixer Lines can not be tested. Please ensure, that they are not in use by any other process!");
				} else {
					boolean compatible = AudioMixer.testMixerCombination(selectedPlaybackMixer, selectedCaptureMixer);
					putNotification(selectedPlaybackMixerComboBox, compatible ? null
							: "The selected mixers can not be run in duplex mode or one of these mixers is already in use by any other process.");
				}
			}
		};
		selectedPlaybackMixerComboBox.addActionListener(mixerTestAction);
		selectedCaptureMixerComboBox.addActionListener(mixerTestAction);

		automaticMixerDetectionCheckBox.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent evt) {
				final boolean autoSelected = automaticMixerDetectionCheckBox.isSelected();
				if (autoSelected) {
					putNotification(selectedPlaybackMixerComboBox, null);
				}
				selectedPlaybackMixerComboBox.setEnabled(!autoSelected);
				selectedCaptureMixerComboBox.setEnabled(!autoSelected);
				AudioMixer mixer = AudioMixer.getInstance();
				if (autoSelected) {
					try {
						mixer.detectMixers();
					} catch (AudioIOException ex) {
						Dialog.message(MixerSettingsDialog.this, "Mixer Auto Detection", ex);
					}
					selectedPlaybackMixerComboBox.setSelectedItem(mixer.getDetectedPlaybackMixer());
					selectedCaptureMixerComboBox.setSelectedItem(mixer.getDetectedCaptureMixer());
				} else {
					try {
						HardwareMixer selectedPlaybackMixer = mixer.getSelectedPlaybackMixer();
						if (selectedPlaybackMixer != null) {
							selectedPlaybackMixerComboBox.setSelectedItem(selectedPlaybackMixer);
						}
						HardwareMixer selectedCaptureMixer = mixer.getSelectedCaptureMixer();
						if (selectedCaptureMixer != null) {
							selectedCaptureMixerComboBox.setSelectedItem(selectedCaptureMixer);
						}
					} catch (AudioIOException ex) {
						Dialog.message(MixerSettingsDialog.this, "Hardware Mixer Selection Error", ex);
					}
				}
			}
		});
		playbackBufferSizeField.getDocument().addDocumentListener(new BufferSizeValidator(playbackBufferSizeField));
		captureBufferSizeField.getDocument().addDocumentListener(new BufferSizeValidator(captureBufferSizeField));
	}

	private class BufferSizeValidator implements DocumentListener {
		JTextField field;

		BufferSizeValidator(JTextField field) {
			this.field = field;
		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		private void checkBufferSize() {
			putNotification(field, null);
			try {
				String text = field.getText();
				int intValue = Integer.parseInt(text.trim());
				// 'guess' the exponent if the assumed value's size is a power
				// of two
				int valueExponent = (int) Math.round(Math.log(intValue) / Math.log(2));
				if (intValue < 2) {
					putNotification(field, "The " + field.getName() + " must be a value greater than one and should be a power of two.");
				} else if (intValue > Math.pow(2, 17)) {
					putNotification(field, "The " + field.getName() + " should be a value lower than 2^17 (" + (int) Math.pow(2, 17) + ").");
				} else if (Math.pow(2, valueExponent) != intValue) {
					// check if the value is a power of two
					putNotification(field, "The " + field.getName() + " should be a power of two (a value of 1024 or 2048 is recommended).");
				}
			} catch (NumberFormatException e) {
				putNotification(field, "The " + field.getName() + " must be a numeric (integral) value greater than one and should be a power of two.");
			}
		}
	}

	private void createLayout() {
		GroupLayout layout = new GroupLayout(mainPanel);
		mainPanel.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		layout.setHorizontalGroup(layout
				.createParallelGroup()
				.addComponent(notificationLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addGroup(
										layout.createParallelGroup().addComponent(automaticMixerDetectionLabel).addComponent(linePreinitializationLabel).addComponent(selectedPlaybackMixerLabel)
												.addComponent(selectedCaptureMixerLabel).addComponent(systemLatencyLabel).addComponent(playbackBufferSizeLabel).addComponent(captureBufferSizeLabel))
								.addGroup(
										layout.createParallelGroup().addComponent(automaticMixerDetectionCheckBox).addComponent(linePreinitializationCheckBox)
												.addComponent(selectedPlaybackMixerComboBox).addComponent(selectedCaptureMixerComboBox).addComponent(systemLatencyField)
												.addComponent(playbackBufferSizeField).addComponent(captureBufferSizeField))));

		layout.setVerticalGroup(layout.createSequentialGroup().addComponent(notificationLabel)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(automaticMixerDetectionLabel).addComponent(automaticMixerDetectionCheckBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(linePreinitializationLabel).addComponent(linePreinitializationCheckBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(selectedPlaybackMixerLabel).addComponent(selectedPlaybackMixerComboBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(selectedCaptureMixerLabel).addComponent(selectedCaptureMixerComboBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(systemLatencyLabel).addComponent(systemLatencyField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(playbackBufferSizeLabel).addComponent(playbackBufferSizeField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(captureBufferSizeLabel).addComponent(captureBufferSizeField))
				// add additional gap to the dialog buttons, because the
				// notification label causes an additional distance to the
				// dialog title on the top
				.addGap(20));
	}

	@SuppressWarnings("unused")
	public static void main(String[] args) {
		GlobalGuiContext.getGuiSettings().initializeUIManager();
		new MixerSettingsDialog(new Object());
		System.exit(0);
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/MixerSettingsDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.model.AudioMixer.getAvailableCaptureMixers;
import static org.audiochain.model.AudioMixer.getAvailablePlaybackMixers;
import static org.audiochain.ui.gui.CommonComponentFactory.createComboBox;
import static org.audiochain.ui.gui.Dialog.isButtonPressed;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.audiochain.io.AudioIOException;
import org.audiochain.model.AudioMixer;
import org.audiochain.model.HardwareMixer;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.GuiSettings;
import org.audiochain.ui.gui.LightEmittingDiodeCheckBox;
import org.audiochain.ui.gui.NotificationDialog;

public class MixerSettingsDialog extends NotificationDialog {
	private JPanel mainPanel;
	private JLabel systemLatencyLabel;
	private JTextField systemLatencyField;
	private JLabel linePreinitializationLabel;
	private JCheckBox linePreinitializationCheckBox;
	private JLabel automaticMixerDetectionLabel;
	private JCheckBox automaticMixerDetectionCheckBox;
	private JLabel selectedPlaybackMixerLabel;
	private JComboBox<HardwareMixer> selectedPlaybackMixerComboBox;
	private JLabel selectedCaptureMixerLabel;
	private JComboBox<HardwareMixer> selectedCaptureMixerComboBox;
	private JLabel playbackBufferSizeLabel;
	private JTextField playbackBufferSizeField;
	private JLabel captureBufferSizeLabel;
	private JTextField captureBufferSizeField;

	public MixerSettingsDialog(Object caller) {
		super(caller);
		init();
	}

	protected void init() {
		if (!Dialog.okCancel(caller, "Mixer Settings Notification", "Please assert that there is no mixer in use by any playback or capture process!")) {
			return;
		}
		
		initComponents();
		initValues();
		registerListeners();
		createLayout();

		if (isButtonPressed(caller, "Audio Mixer Settings", mainPanel, applyButton)) {
			AudioMixer mixer = AudioMixer.getInstance();
			boolean autoDetection = automaticMixerDetectionCheckBox.isSelected();
			mixer.setAutomaticMixerDetection(autoDetection);
			mixer.setLinePreinitialization(linePreinitializationCheckBox.isSelected());
			mixer.setSystemLatencyInFrames(Integer.parseInt(systemLatencyField.getText().trim()));
			mixer.setPlaybackBufferSizeInSamples(Integer.parseInt(playbackBufferSizeField.getText().trim()));
			mixer.setCaptureBufferSizeInSamples(Integer.parseInt(captureBufferSizeField.getText().trim()));
			if (!autoDetection) {
				mixer.setSelectedPlaybackMixer((HardwareMixer) selectedPlaybackMixerComboBox.getSelectedItem());
				mixer.setSelectedCaptureMixer((HardwareMixer) selectedCaptureMixerComboBox.getSelectedItem());
			}
			mixer.store();
		}
	}

	private void initComponents() {
		mainPanel = new JPanel();
		mainPanel.setOpaque(false);
		applyButton = new JButton("Apply Changes");
		systemLatencyLabel = new JLabel("General System Latency (Frames):");
		systemLatencyField = new JTextField(4);
		linePreinitializationLabel = new JLabel("Initialize Audio Lines On Startup:");
		linePreinitializationCheckBox = new LightEmittingDiodeCheckBox();
		automaticMixerDetectionLabel = new JLabel("Automatic Mixer Detection:");
		automaticMixerDetectionCheckBox = new LightEmittingDiodeCheckBox();
		selectedPlaybackMixerLabel = new JLabel("Playback Mixer:");
		selectedPlaybackMixerComboBox = createComboBox(getAvailablePlaybackMixers());
		selectedCaptureMixerLabel = new JLabel("Recording Mixer:");
		selectedCaptureMixerComboBox = createComboBox(getAvailableCaptureMixers());
		playbackBufferSizeLabel = new JLabel("Playback Buffer Size (Samples):");
		playbackBufferSizeField = new JTextField(5);
		playbackBufferSizeField.setName("Playback Buffer Size");
		captureBufferSizeLabel = new JLabel("Recording Buffer Size (Samples):");
		captureBufferSizeField = new JTextField(5);
		captureBufferSizeField.setName("Capture Buffer Size");
	}

	private void initValues() {
		AudioMixer mixer = AudioMixer.getInstance();
		systemLatencyField.setText(String.valueOf(mixer.getSystemLatencyInFrames()));
		linePreinitializationCheckBox.setSelected(mixer.isLinePreinitialization());
		boolean automaticMixerDetection = mixer.isAutomaticMixerDetection();
		automaticMixerDetectionCheckBox.setSelected(automaticMixerDetection);

		if (automaticMixerDetection) {
			selectedPlaybackMixerComboBox.setSelectedItem(mixer.getDetectedPlaybackMixer());
			selectedCaptureMixerComboBox.setSelectedItem(mixer.getDetectedCaptureMixer());
		} else {
			try {
				HardwareMixer selectedPlaybackMixer = mixer.getSelectedPlaybackMixer();
				if (selectedPlaybackMixer != null) {
					selectedPlaybackMixerComboBox.setSelectedItem(selectedPlaybackMixer);
				}
				HardwareMixer selectedCaptureMixer = mixer.getSelectedCaptureMixer();
				if (selectedCaptureMixer != null) {
					selectedCaptureMixerComboBox.setSelectedItem(selectedCaptureMixer);
				}
			} catch (AudioIOException ex) {
				Dialog.message(MixerSettingsDialog.this, "Hardware Mixer Selection Error", ex);
			}
		}

		selectedPlaybackMixerComboBox.setEnabled(!automaticMixerDetectionCheckBox.isSelected());
		selectedCaptureMixerComboBox.setEnabled(!automaticMixerDetectionCheckBox.isSelected());

		playbackBufferSizeField.setText(String.valueOf(mixer.getPlaybackBufferSizeInSamples()));
		captureBufferSizeField.setText(String.valueOf(mixer.getCaptureBufferSizeInSamples()));
	}

	private void registerListeners() {
		systemLatencyField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				checkSystemLatencyValue();
			}

			private void checkSystemLatencyValue() {
				putNotification(systemLatencyField, null);
				try {
					String text = systemLatencyField.getText();
					Integer.parseInt(text.trim());
				} catch (NumberFormatException e) {
					putNotification(systemLatencyField, systemLatencyLabel.getText() + " must have a numeric (integral) value: " + e.getMessage());
				}
			}
		});

		ActionListener mixerTestAction = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (automaticMixerDetectionCheckBox.isSelected()) {
					putNotification(selectedPlaybackMixerComboBox, null);
					return;
				}
				HardwareMixer selectedPlaybackMixer = (HardwareMixer) selectedPlaybackMixerComboBox.getSelectedItem();
				HardwareMixer selectedCaptureMixer = (HardwareMixer) selectedCaptureMixerComboBox.getSelectedItem();
				if (selectedPlaybackMixer == null || selectedCaptureMixer == null) {
					putNotification(selectedPlaybackMixerComboBox, "The Mixer Lines can not be tested. Please ensure, that they are not in use by any other process!");
				} else {
					boolean compatible = AudioMixer.testMixerCombination(selectedPlaybackMixer, selectedCaptureMixer);
					putNotification(selectedPlaybackMixerComboBox, compatible ? null
							: "The selected mixers can not be run in duplex mode or one of these mixers is already in use by any other process.");
				}
			}
		};
		selectedPlaybackMixerComboBox.addActionListener(mixerTestAction);
		selectedCaptureMixerComboBox.addActionListener(mixerTestAction);

		automaticMixerDetectionCheckBox.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent evt) {
				final boolean autoSelected = automaticMixerDetectionCheckBox.isSelected();
				if (autoSelected) {
					putNotification(selectedPlaybackMixerComboBox, null);
				}
				selectedPlaybackMixerComboBox.setEnabled(!autoSelected);
				selectedCaptureMixerComboBox.setEnabled(!autoSelected);
				AudioMixer mixer = AudioMixer.getInstance();
				if (autoSelected) {
					try {
						mixer.detectMixers();
					} catch (AudioIOException ex) {
						Dialog.message(MixerSettingsDialog.this, "Mixer Auto Detection", ex);
					}
					selectedPlaybackMixerComboBox.setSelectedItem(mixer.getDetectedPlaybackMixer());
					selectedCaptureMixerComboBox.setSelectedItem(mixer.getDetectedCaptureMixer());
				} else {
					try {
						HardwareMixer selectedPlaybackMixer = mixer.getSelectedPlaybackMixer();
						if (selectedPlaybackMixer != null) {
							selectedPlaybackMixerComboBox.setSelectedItem(selectedPlaybackMixer);
						}
						HardwareMixer selectedCaptureMixer = mixer.getSelectedCaptureMixer();
						if (selectedCaptureMixer != null) {
							selectedCaptureMixerComboBox.setSelectedItem(selectedCaptureMixer);
						}
					} catch (AudioIOException ex) {
						Dialog.message(MixerSettingsDialog.this, "Hardware Mixer Selection Error", ex);
					}
				}
			}
		});
		playbackBufferSizeField.getDocument().addDocumentListener(new BufferSizeValidator(playbackBufferSizeField));
		captureBufferSizeField.getDocument().addDocumentListener(new BufferSizeValidator(captureBufferSizeField));
	}

	private class BufferSizeValidator implements DocumentListener {
		JTextField field;

		BufferSizeValidator(JTextField field) {
			this.field = field;
		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			checkBufferSize();
		}

		private void checkBufferSize() {
			putNotification(field, null);
			try {
				String text = field.getText();
				int intValue = Integer.parseInt(text.trim());
				// 'guess' the exponent if the assumed value's size is a power
				// of two
				int valueExponent = (int) Math.round(Math.log(intValue) / Math.log(2));
				if (intValue < 2) {
					putNotification(field, "The " + field.getName() + " must be a value greater than one and should be a power of two.");
				} else if (intValue > Math.pow(2, 17)) {
					putNotification(field, "The " + field.getName() + " should be a value lower than 2^17 (" + (int) Math.pow(2, 17) + ").");
				} else if (Math.pow(2, valueExponent) != intValue) {
					// check if the value is a power of two
					putNotification(field, "The " + field.getName() + " should be a power of two (a value of 1024 or 2048 is recommended).");
				}
			} catch (NumberFormatException e) {
				putNotification(field, "The " + field.getName() + " must be a numeric (integral) value greater than one and should be a power of two.");
			}
		}
	}

	private void createLayout() {
		GroupLayout layout = new GroupLayout(mainPanel);
		mainPanel.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		layout.setHorizontalGroup(layout
				.createParallelGroup()
				.addComponent(notificationLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addGroup(
										layout.createParallelGroup().addComponent(automaticMixerDetectionLabel).addComponent(linePreinitializationLabel).addComponent(selectedPlaybackMixerLabel)
												.addComponent(selectedCaptureMixerLabel).addComponent(systemLatencyLabel).addComponent(playbackBufferSizeLabel).addComponent(captureBufferSizeLabel))
								.addGroup(
										layout.createParallelGroup().addComponent(automaticMixerDetectionCheckBox).addComponent(linePreinitializationCheckBox)
												.addComponent(selectedPlaybackMixerComboBox).addComponent(selectedCaptureMixerComboBox).addComponent(systemLatencyField)
												.addComponent(playbackBufferSizeField).addComponent(captureBufferSizeField))));

		layout.setVerticalGroup(layout.createSequentialGroup().addComponent(notificationLabel)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(automaticMixerDetectionLabel).addComponent(automaticMixerDetectionCheckBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(linePreinitializationLabel).addComponent(linePreinitializationCheckBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(selectedPlaybackMixerLabel).addComponent(selectedPlaybackMixerComboBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(selectedCaptureMixerLabel).addComponent(selectedCaptureMixerComboBox))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(systemLatencyLabel).addComponent(systemLatencyField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(playbackBufferSizeLabel).addComponent(playbackBufferSizeField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(captureBufferSizeLabel).addComponent(captureBufferSizeField))
				// add additional gap to the dialog buttons, because the
				// notification label causes an additional distance to the
				// dialog title on the top
				.addGap(20));
	}

	@SuppressWarnings("unused")
	public static void main(String[] args) {
		GuiSettings.getInstance().initializeUIManager();
		new MixerSettingsDialog(new Object());
		System.exit(0);
	}
}
